/*
 * AVR based RGB Moodlight
 *
 * moodlight.c - moodlight modes and remote control
 *
 * (c) Axel (XL) Schwenke, axel.schwenke@gmx.net
 *
 * $Id: moodlight.c 152 2012-03-08 08:32:46Z schwenke $
 
 
Version3 soll auf dem AT90S4433 portiert werden. Hierzu bentigt die Schaltung noch einen Quarz 8 MHz und zwei Kondensatoren 33 pF.
 
 */
 
 
#include <avr/io.h>		//von Chr hinzugefgt wegen den PORT-Pins
#include <inttypes.h>

//#include "irmp_chr2.h"
#include "moodlight_chr5_3.h"
#include <util/delay.h>		//nur fr die delay-Funktion: Temperaturanzeige bei Tastendruck auf 0.


//angepasst auf liebliongs-RC5-Fernebdienung
//#define A_KEY_OPERATE 0x0C                /* on/off 	bringt Programm zum Absturz, zhlen geht ganz langsam*/	
#define A_KEY_OPERATE 0x0A                /* on/off 	bringt Programm zum Absturz, zhlen geht ganz langsam*/	
#define A_KEY_1       0x01                /* mode 1 */		/*Taste defekt -- Taste 4 verwendet*/
#define A_KEY_2       0x02                /* mode 2 */
#define A_KEY_3       0x03                /* mode 3 */
#define A_KEY_4       0x04                /* mode 1 */
#define A_KEY_5       0x05                /* mode 2 */
#define A_KEY_6       0x06                /* mode 3 */
#define A_KEY_7       0x07					//Taste fr Einschalten, da die Aus-Taste niemlas wieder einschaltet
#define A_KEY_8       0x08
#define A_KEY_9       0x09
#define A_KEY_0       0x00				//Temperaturanzeige auslsen
#define A_KEY_MINUS   0x15
#define A_KEY_PLUS    0x16
#define A_KEY_REV     0x15                /* Pfeil rechts Taste slower */
#define A_KEY_FWD     0x16                /* Pfeil links Taste faster */
#define A_KEY_STOP    0x0D					//Lautsprecher   ohne Wirkung
#define A_KEY_PLAY    0x23                /* | || -Taste change direction  OK, funktioniert */
//#define A_KEY_REC     0x09
#define A_KEY_PAUSE   0x35                /* OK-Taste stop/start OK funktioniert*/



/* bits in ml.flags */

#define ML_FLAG_OFF   0x80
#define ML_FLAG_PAUSE 0x02
#define ML_FLAG_UP    0x01

/* more moodlight constants */

#define ML_MODE3_PERIOD 0x40  /* must be a power of 2 */

/* global moodlight variables(s) */

struct {
    uint8_t  mode;      /* the active moodlight mode */
    uint8_t  delay;     /* run moodlight_step() every "delay" PWM cycles */
    uint16_t cycle;     /* counter used by modes */
    uint8_t  flags;     /* on/off, stopped, direction */
} ml;

/* led brightness (0..MAX_BRIGHT) */

uint8_t level_r;
uint8_t level_g;
uint8_t level_b;

/* clock tick, decremented once in each PWM cycle */

volatile uint8_t tick;


/* clock tick, for temperature-measurement */
volatile uint8_t tick_timer0 = 0; 	//fr timer0-Interrupt fr DS18S20 Temperaturmessung

/* last received IR signal */




/* evaluate an IR remote signal */
void moodlight_remote(void)
{
//Dieser Teil ist fr alle vier Fernbedienungen:
    switch ((i & 0x3F) | (~i >> 7 & 0x40)) {					//ndern<<<<<<<-------<<<<<<<<<<------------<<<<<<<

//Dieser Teil ist fr die erste Fernbedienung:
    case A_KEY_OPERATE: /* on/off */			//schaltet ab, aber nicht mehr ein. Hauptschleife wird nur noch selten durchlaufen. Deshalb zu seltener RC5-Empfang.
        {
                ml.flags ^= ML_FLAG_OFF;			//Exor -->> toggeln des Bits
                if (ml.flags & ML_FLAG_OFF) {
                    level_r = 0;
                    level_g = 0;
                    level_b = 0;
                } else {
                    moodlight_wakeup();
            
            }
            break;
        }

    case A_KEY_0:		//Temperaturanzeige auslsen, wirkt fr NEC und RC5-FB, weil beide 0x00 bei Taste 0 senden.
        {
      		/*
			TIMSK1 = 0;	// timer1
			start_meas();	//das erste der beiden Unterprogramme in TEMPMEAS.c
			TIMSK1 = _BV(OCIE1A);	// timer1
			//_delay_ms(700);		//geht auch ohne delay
			TIMSK1 = 0;	// timer1
			//read_meas();			//das zweite der beiden Unterprogramme in TEMPMEAS.c
			read_meas_lcd();		//ruft die zweite LESE-Routine auf, die die LCD-Ausgabe enthlt.
			TIMSK1 = _BV(OCIE1A);	// timer1
			*/
			break;
        }


    case A_KEY_4:
        {
            ml.mode = 1;
            ml.delay = 6;
            ml.flags = 0;
            ml.cycle = 1;
            tick = 1;
            break;
        }

    case A_KEY_5:
        {
            ml.mode = 2;
            ml.delay = 4;
            ml.flags = 0;
            ml.cycle = 1;
            tick = 1;
            break;
        }

    case A_KEY_6:
        {
            ml.mode = 3;
            ml.delay = 4;
            ml.flags = 0;
            ml.cycle = 0;
            tick = 1;
            break;
        }

    case A_KEY_FWD: /* faster */
        {
            if ((ml.mode == 1 || ml.mode == 2 || ml.mode == 3)
                && ml.delay > 1) {
                ml.delay--;							//zhlt bei jedem richtigen Tastendruck eins weiter bis 0
            }
            break;
        }

    case A_KEY_REV: /* slower */
        {
            if (ml.mode == 1 && ml.delay < 32) {
                ml.delay++;								//zhlt bei jedem richtigen Tastendruck eins weiter 0..32 oder 0...20 
            }
            if ((ml.mode == 2 || ml.mode == 3) && ml.delay < 20) {	//
                ml.delay++;
            }
            break;
        }

    case A_KEY_PAUSE: /* stop motion */
        {
                ml.flags ^= ML_FLAG_PAUSE;				//OK, Farbe ndert sich nicht mehr
            
            break;
        }

    case A_KEY_PLAY:  /* start motion / change direction */		//OK, funktioniert
        {
                if (ml.flags & ML_FLAG_PAUSE) {
                    ml.flags ^= ML_FLAG_PAUSE;
                } else {
                    ml.flags ^= ML_FLAG_UP;
                }
            
            break;
        }
/*
//neu hinzu gefgt: STOP-Taste hat gefehlt
    case A_KEY_STOP: // stop motion 
        {
                ml.flags ^= ML_FLAG_PAUSE;
            
            break;
        }
*/	
	
//neu hinzu gefgt: Ein/aus-Taste geht nicht
	case A_KEY_7:
		{		
				ml.flags = 0;
                //ml.flags &= ~ML_FLAG_OFF;		//das einzelne Bit Nr7 lschen
				//moodlight_wakeup();
				moodlight_init();				// geht nicht
				setup_pwm();					// geht nicht
				break;				
		}

		

    }
}


/* initialization */
void moodlight_init(void)
{
    ml.mode  = 1;
    ml.delay = 6;
    ml.flags = 0;
    ml.cycle = 1;
    tick = 1;
}


/* called after waking up from power-off */
void moodlight_wakeup(void)
{
    switch (ml.mode) {
    case 3:
        {
            ml.cycle &= ~(ML_MODE3_PERIOD-1);
            break;
        }
    }
    tick = 1;
}


/* called after last pwm phase when tick == 0 */
void moodlight_step(void)
{

#ifdef use_uart	

/*	itoa( ml.mode, s, 10);		//Anzeige der eingestellten Betriebsart														binr nach ASCII-Umwandlung
	uputsnl(s);					//funktioniert, schreibt die Zahl, danach Zeilenvorschub	
	itoa( ml.delay, s, 10);	//Anzige der eingestellten Geschwindigleit. am schnellsten: 1, am langsamsten:32			binr nach ASCII-Umwandlung
	uputsnl(s);					//funktioniert, schreibt die Zahl, danach Zeilenvorschub
	itoa( ml.cycle, s, 10);	//zhlt abwrts oder aufwrts je nach eingestellter Richtung und unterschiedlich schnell	binr nach ASCII-Umwandlung
	uputsnl(s);					//funktioniert, schreibt die Zahl, danach Zeilenvorschub
*/
/*
Nach "A_KEY_PAUSE" bleibt UART-ausgabe komplett weg. Nein, scheint nur stehen zu bleiben im Terminal, wird aber stndig weiter geschrieben.
Nach "A_KEY_PLAY" change Direction bestimmt, ob "ml.cycle" aufwrts oder abwrts zhlt. Voreinstellung ist abwrts.


Auswertung und Anzeige der eingestellten Flags:
#define ML_FLAG_OFF   0x80
#define ML_FLAG_PAUSE 0x02
#define ML_FLAG_UP    0x01
OK, die Anzeige Funktioniert, aber das Licht ist total gestrt. UART-Ausgabe dauert zu lange. Abhilfe: Einige Ausgaben entfernen.
*/

//	if (ml.flags == ML_FLAG_UP)		uputs( "UP " );	
//	if (ml.flags == ML_FLAG_PAUSE)	uputs( "Pause " );	
//	if (ml.flags == ML_FLAG_OFF)	uputs( "OFF " );	
//	if (ml.flags == ML_FLAG_OFF)	uputs( "OFF \n\r" );	
//	uputs( "\n\r" );	
#endif



/*
Ohne diese erste IF-Abfrage kann man im ausgeschalteten Zustand schn das gesetzte Flag am Terminal beobachten. Das Licht leuchtet weiter und ndert sich.
Es lt sich tatschlich wieder "einschalten" und das Flag verschwindet. Bei gesetztem flag lt sich trotzdem die Geschwindigkeit verndern.
Die erste IF-Abfrage dient dazu, das gesamte Unterprogramm zu verlassen, bevor die anderen Anweisungen ausgefhrt werden knnen, falls abgeschaltet ist.

*/
    if (ml.flags & ML_FLAG_OFF) {
        return;
    }

    switch (ml.mode) {

    case 1: /* fading R-G-B */
        {
            if (ml.cycle <= MAX_BRIGHT) {
                level_r = (MAX_BRIGHT+1) - ml.cycle;
                level_g = ml.cycle;
                level_b = 0;
            }
            else if (ml.cycle <= 2*MAX_BRIGHT) {
                level_r = 0;
                level_g = (2*MAX_BRIGHT+1) - ml.cycle;
                level_b = ml.cycle - MAX_BRIGHT;
            }
            else {
                level_r = ml.cycle - (2*MAX_BRIGHT);
                level_g = 0;
                level_b = (3*MAX_BRIGHT+1) - ml.cycle;
            }

            if ((ml.flags & ML_FLAG_PAUSE) == 0) {
                if (ml.flags & ML_FLAG_UP) {										//conditional Operator siehe
                    ml.cycle = (ml.cycle == 3*MAX_BRIGHT+1 ? 1 : ml.cycle+1);		//http://gd.tuwien.ac.at/languages/c/cref-mleslie/SYNTAX/conditional.html
                } else {															//durchluft 0...612 = 3 x 204
                    ml.cycle = (ml.cycle == 0 ? 3*MAX_BRIGHT : ml.cycle-1);;		//durchluft 612...0 = 3 x 204
                }
            }
            tick = ml.delay;
            break;
        }

    case 2: /* fading R-R+G-G-G+B-B-B+R */
        {
            if (ml.cycle <= MAX_BRIGHT) {
                /* R -> R+G */
                level_r = MAX_BRIGHT;
                level_g = ml.cycle;
                level_b = 0;
            }
            else if (ml.cycle <= 2*MAX_BRIGHT) {
                /* R+G -> G */
                level_r = 2*MAX_BRIGHT+1-ml.cycle;
                level_g = MAX_BRIGHT;
                level_b = 0;
            }
            else if (ml.cycle <= 3*MAX_BRIGHT) {
                /* G -> G+B */
                level_r = 0;
                level_g = MAX_BRIGHT;
                level_b = ml.cycle - 2*MAX_BRIGHT;
            }
            else if (ml.cycle <= 4*MAX_BRIGHT) {
                /* G+B -> B */
                level_r = 0;
                level_g = 4*MAX_BRIGHT+1 - ml.cycle;
                level_b = MAX_BRIGHT;
            }
            else if (ml.cycle <= 5*MAX_BRIGHT) {
                /* B -> B+R */
                level_r = ml.cycle - 4*MAX_BRIGHT;
                level_g = 0;
                level_b = MAX_BRIGHT;
            }
            else {
                /* B+R -> R */
                level_r = MAX_BRIGHT;
                level_g = 0;
                level_b = 6*MAX_BRIGHT+1 - ml.cycle;
            }

            if ((ml.flags & ML_FLAG_PAUSE) == 0) {
                if (ml.flags & ML_FLAG_UP) {
                    ml.cycle = (ml.cycle == 6*MAX_BRIGHT+1 ? 1 : ml.cycle+1);		//durchluft 0...1224 = 6 x 204
                } else {
                    ml.cycle = (ml.cycle == 0 ? 6*MAX_BRIGHT : ml.cycle-1);;		//durchluft 1224...0 = 6 x 204
                }
            }
            tick = ml.delay;
            break;
        }

    case 3: /* switching R-R+G-G-G+B-B-R+B-R+G+B */
        {
            if ((ml.cycle & (ML_MODE3_PERIOD-1)) == 0) {
                switch (ml.cycle/ML_MODE3_PERIOD) {									//Division
                case 0:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = 0;
                        level_b = 0;
                        break;
                    }
                case 1:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = MAX_BRIGHT;
                        level_b = 0;
                        break;
                    }
                case 2:
                    {
                        level_r = 0;
                        level_g = MAX_BRIGHT;
                        level_b = 0;
                        break;
                    }
                case 3:
                    {
                        level_r = 0;
                        level_g = MAX_BRIGHT;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                case 4:
                    {
                        level_r = 0;
                        level_g = 0;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                case 5:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = 0;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                case 6:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = MAX_BRIGHT;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                }
            }

            if ((ml.flags & ML_FLAG_PAUSE) == 0) {
                if (ml.flags & ML_FLAG_UP) {
                    ml.cycle = (ml.cycle == (7*ML_MODE3_PERIOD-1) ? 0 : ml.cycle+1);		//durchluft 0...441		ML_MODE3_PERIOD = 0x40 = 64 dezimal
                } else {
                    ml.cycle = (ml.cycle == 0 ? (7*ML_MODE3_PERIOD-1) : ml.cycle-1);;		//durchluft 441...0		7 x (64 -1) = 441
                }																			
            }
            tick = ml.delay;
            break;
        }

    }
}

/*
ich hab in der Zwischenzeit ein bisschen am Code gefeilt und jetzt 3 
Modi implementiert:

Mode 1: sanfter Farbwechsel R->G->B->R. Die Farben "treffen" sich bei 
50% Intensitt, dadurch sind die Zwischenfarben (z.B. Magenta = 
Rot+Blau) etwas dunkler als die reinen Farben.

Mode 2: sanfter Farbwechsel R->R+G->G->G+B->B->B+R->R. Hier haben die 
LED bei den Zwischenfarben jeweils 100% Intensitt, wodurch jetzt die 
Zwischenfarben heller erscheinen.

Mode 3: hartes Umschalten zwischen R, R+G, G, G+B, B, B+R, R+G+B.

In allen 3 Modi kann die Geschwindigkeit mit 2 Tasten (FWD, REV) 
verndert werden. Auerdem kann man das Moodlight mit einer Taste 
(PAUSE) anhalten (und mit einem weiteren Druck weiter laufen) lassen. 
Schlielich kann man mit einer weiteren Taste (PLAY) entweder aus dem 
Pause-Zustand weiterlaufen oder wenn es schon luft, die Richtung 
wechseln. Zu guter Letzt kann man mit einer Taste (OPERATE) das 
Moodlight ausschalten. Der C bleibt unter Strom, so da man mit der 
Fernbedienung wieder einschalten kann. Die Einstellungen bleiben 
erhalten.

Der Code ist jetzt noch aufgerumter und besser kommentiert. Auerdem 
habe ich das fr meinen Prototypen verwendete Platinenlayout dazu 
gepackt.

In den nchsten Tagen werde ich den Code auf einen ATtiny85 portieren 
und die finale Hardware bauen. Der Prototyp sieht schon sehr nett aus, 
nachdem ich die Milchglaskugel einer "Fado" Lampe (von IKEA: 
http://www.ikea.com/de/de/catalog/products/80096372/) drber gestlpt 
habe.


In der Hoffnung, jemanden zu inspirieren :)

*/
